# read in stats from python
# and visualize


library(foreach)
library(data.table)
library(raster)
library(lubridate)
library(magrittr)
library(ggplot2)
library(forcats)

ncell <- 14051167

## read/prep data ####

all_files <- list.files("data-orig/freq-v3", full.names = T)
dat_stats <- foreach(
  i_file = all_files,
  .final = rbindlist
) %do% {
  
  dat <- fread(i_file)
  
  dat[, .(ff = basename(i_file) %>% gsub(".csv", "", .),
          date = as.Date(ymd_hms(file)),
          value, 
          count)]

}

# summarize nodata/water into 1 class

dat_stats[, count_frac := count / ncell]
dat_stats[, value_f := fct_collapse(as.character(value),
                                    nodata_water = c("0", "4", "5"),
                                    snow = "1",
                                    land = "2",
                                    clouds = "3")]
dat_stats_class <- dat_stats[, 
                             .(count_frac = sum(count_frac)),
                             .(ff, date, value_f)]

# fill in 0's for missing dates 

CJ(ff = unique(dat_stats_class$ff),
   value_f = unique(dat_stats_class$value_f),
   date = seq(min(dat_stats$date), max(dat_stats$date), by = "day")) %>% 
  merge(dat_stats_class, all.x = T) -> dat_stats_class_full
# with(dat_stats_class_full, table(ff, value_f, is.na(count_frac)))

dat_stats_class_full[is.na(count_frac), count_frac := 0]

## monhtly average clouds ####

dat_stats_class_full[value_f == "clouds" & ff != "05_temporal_complete_max99d", 
                     .(count_frac = mean(count_frac)),
                     .(ff, month(date), value_f)] -> dat_plot

dat_plot[, hydro_month := factor(month.abb[month], levels = month.abb[c(10:12, 1:9)])]

cols <- scales::hue_pal()(6)
names(cols) <- sort(unique(dat_plot$ff))

gg1 <- dat_plot %>% 
  ggplot(aes(hydro_month, count_frac, group = ff, colour = ff))+
  geom_point()+
  geom_line()+
  scale_colour_manual("", values = cols)+
  scale_y_continuous(labels = scales::percent_format())+
  theme_bw()+
  xlab(NULL)+
  ylab("Average cloud fraction per image")

gg1_leg <- cowplot::get_legend(gg1)

gg2 <- dat_plot[startsWith(ff, "05")] %>% 
  ggplot(aes(hydro_month, count_frac, group = ff, colour = ff))+
  geom_point()+
  geom_line()+
  scale_colour_manual("", values = cols)+
  scale_y_continuous(labels = scales::percent_format())+
  theme_bw()+
  theme(legend.position = "none")+
  xlab(NULL)+
  ylab("Average cloud fraction per image")

gg_all <- cowplot::plot_grid(gg1 + theme(legend.position = "none"), gg2, gg1_leg, 
                             nrow = 1, labels = c("A", "B", ""), rel_widths = c(1, 1, .6))

ggsave(gg_all, filename = "fig/cloud-fraction-sequence.png",
       width = 12, height = 4, units = "in")


## boxplots cloud fractions by step ####
# 
# gg <- dat_stats[value == 3] %>% 
#   ggplot(aes(fct_rev(ff), count_frac))+
#   geom_boxplot()+
#   scale_y_continuous(labels = scales::percent_format())+
#   coord_flip()+
#   theme_bw()+
#   xlab("Cloud removal step")+
#   ylab("Cloud fraction per image")
# 
# 
# ggsave(gg,
#        file = "fig/cloud-fraction-sequence-boxplot.png",
#        width = 6,
#        height = 4)




## histogram cloud fractions by step ####

# dat_stats_class_full[value_f == "clouds",
#                      .N,
#                      ff]

arrow_text_color <- "white"

gg01_04 <- dat_stats_class_full[value_f == "clouds" & ! startsWith(ff, "05")] %>% 
  ggplot(aes(count_frac))+
  geom_histogram(boundary = 0)+
  scale_x_continuous(labels = scales::percent_format())+
  facet_wrap(~ ff, ncol = 1)+
  theme_bw()+
  xlab("Cloud fraction per image")+
  ylab("Count")


# manual 10d
l_hist_10d <- hist(dat_stats_class_full[value_f == "clouds" & ff == "05_temporal_complete_max10d",
                                        count_frac],
                   plot = F, breaks = 30)

dat_hist_10d <- data.table(mids = l_hist_10d$mids, 
                           counts = l_hist_10d$counts,
                           ff = "05_temporal_complete_max10d")

dat_hist_10d[, yy := counts]
dat_hist_10d[1, yy := NA]
y_first_bar_10d <- dat_hist_10d[1, counts]

gg05_10d <- dat_hist_10d %>% 
  ggplot(aes(mids, yy))+
  geom_bar(width = 0.005, stat = "identity")+
  scale_x_continuous(labels = scales::percent_format())+
  facet_wrap(~ ff, ncol = 1, scales = "free")+
  theme_bw()+
  xlab("Cloud fraction per image")+ 
  ylab("Count")+
  annotate("rect", xmin = 0, xmax = 0.005, ymin = 0, ymax = Inf)+
  annotate("segment", x = 0.0025, xend = 0.0025, y = 250, yend = 300,
           arrow = arrow(type = "open", length = unit(0.15, "inches")), 
           colour = arrow_text_color, arrow.fill = arrow_text_color)+
  annotate("text", 
           x = 0.0025, y = 240, label = paste0("count = ", y_first_bar_10d), 
           hjust = 1.1, vjust = 0.5, angle = 90, colour = arrow_text_color)





gg_final <- cowplot::plot_grid(gg01_04, gg05_10d, 
                   nrow = 1, labels = c("A", "B"))

ggsave(width = 12, height = 8, units = "in",
      filename = "fig/cloud-fraction-sequence-histogram.png",
      plot = gg_final)


## save data ####

saveRDS(dat_stats_class_full, "data/sequence-stats-v04.rds")

## EOF ####
