# compare different bias correction routines for subset of rcm
# stepsize: 1 pixel, 1 month

# delta change method (one delta per month and pixel)

library(data.table)
library(magrittr)
library(eurocordexr)
library(foreach)
library(lubridate)
library(fs)
library(raster)

mitmatmisc::init_parallel_ubuntu(6)


# parameters --------------------------------------------------------------

path_out <- "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/bc_full/01_delta_change/"
dir_create(path_out)

# set 0 below this threshold [snc = 0-100]
min_snc <- 0.001

# data --------------------------------------------------------------------

dat_th <- readRDS("data/auxiliary-02-snow-max-threshold.rds")

dat_modis <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/rds/modis-10d-02-snc.rds")
dat_modis[, month := month(date)]
dat_modis <- dat_modis[!is.na(snc)]

# prep rcm data 

dat_inv_past <- get_inventory("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/nc_daily/rcm/2000-2020/", add_files = T)
dat_inv_future <- get_inventory("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/nc_daily/rcm/2071-2100/", add_files = T)

# no HIRHAM
dat_loop_rcm <- dat_inv_future[institute_rcm != "DMI-HIRHAM5",
                               .SD, 
                               institute_rcm,
                               .SDcols = c("gcm", "experiment", "ensemble", "downscale_realisation")]

# main loop ---------------------------------------------------------------


# i_rcm <- 2

foreach(
  i_rcm = 1:nrow(dat_loop_rcm),
  .inorder = F
) %dopar% {
    
  dat_loop_rcm[i_rcm] %>% 
    unlist %>% 
    paste0(collapse = "_") -> fn_rcm
  fn_out <- path(path_out, fn_rcm, ext = "rds")
  if(file_exists(fn_out)) return(paste0("Skipped: ", fn_out))
  
  
  # past data
  dat_inv_past %>% 
    merge(dat_loop_rcm[i_rcm], by = names(dat_loop_rcm)) %>% 
    .$list_files %>% 
    .[[1]] -> files_past1
  dat_inv_past[experiment == "historical"] %>% 
    merge(dat_loop_rcm[i_rcm], by = names(dat_loop_rcm)[names(dat_loop_rcm) != "experiment"]) %>% 
    .$list_files %>% 
    .[[1]] -> files_past2
  files_past <- c(files_past1, files_past2)
  
  dat_past <- foreach(i_fn = files_past, .final = rbindlist) %do% {
    nc_grid_to_dt(i_fn, "snc", interpolate_to_standard_calendar = T)
  }
  
  # future data
  dat_inv_future %>% 
    merge(dat_loop_rcm[i_rcm], by = names(dat_loop_rcm)) %>% 
    .$list_files %>% 
    .[[1]] -> files_future
  
  dat_future <- foreach(i_fn = files_future, .final = rbindlist) %do% {
    nc_grid_to_dt(i_fn, "snc", interpolate_to_standard_calendar = T)
  }
  
  # remove cells with implausible snow (accumulation) & 
  # subset to common cells (modis & rcm)
  icell_rcm <- dat_th[rcm == dat_loop_rcm[i_rcm, institute_rcm] & snow_plausible == "yes", icell]
  icell_modis <- unique(dat_modis$icell)
  icell_yes <- icell_rcm[icell_rcm %in% icell_modis]
  
  dat_past <- dat_past[icell %in% icell_yes]
  dat_future <- dat_future[icell %in% icell_yes]
  
  # other req data
  setkey(dat_past, icell, date)
  setkey(dat_future, icell, date)
  
  dat_past[, month := month(date)]
  dat_future[, month := month(date)]
  
  dat_past[, snc := snc/100]
  dat_future[, snc := snc/100]
  
  dat_past_avg <- dat_past[, .(snc_past = mean(snc)), .(icell, month)]
  dat_future_avg <- dat_future[, .(snc_future = mean(snc)), .(icell, month)]
  
  dat_delta <- merge(dat_past_avg, dat_future_avg)
  
  dat_delta[snc_past < min_snc, snc_past := 0]
  dat_delta[snc_future < min_snc, snc_future := 0]
  
  dat_delta[, delta_snc_mult := snc_future/snc_past]
  dat_delta[, delta_snc_add := snc_future - snc_past]

  dat_delta[is.infinite(delta_snc_mult), delta_snc_mult := NA]  
  
  # dat_delta %>% summary
  # dat_delta[is.na(delta_snc_mult)] %>% summary
  # dat_delta[is.infinite(delta_snc_mult)] %>% summary
  # dat_delta[delta_snc_mult > 1] %>% summary
  
  
  # save results
  saveRDS(dat_delta, fn_out)

  return(paste0("Done: ", fn_out))
  
  
  
}

