# look at modis gapfill stats

library(foreach)
library(data.table)
library(raster)
library(lubridate)
library(magrittr)
library(ggplot2)
library(forcats)

rr <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/05_temporal_complete_max10d/20000224_120000.tif")
ncell <- ncell(rr)



## read/prep data ####

all_files <- list.files("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/stats/", 
                        full.names = T, pattern = "^[^reg]")

dat_stats <- foreach(
  i_file = all_files,
  .final = rbindlist
) %do% {
  
  dat <- fread(i_file)
  
  dat[, .(ff = basename(i_file) %>% gsub(".csv", "", .),
          date = as.Date(ymd_hms(file)),
          value, 
          count)]
  
}

# summarize nodata/water into 1 class

dat_stats[, count_frac := count / ncell]
dat_stats[, value_f := fct_collapse(as.character(value),
                                    nodata_water = c("0", "4", "5"),
                                    snow = "1",
                                    land = "2",
                                    clouds = "3")]
dat_stats_class <- dat_stats[, 
                             .(count_frac = sum(count_frac),
                               count_sum = sum(count)),
                             .(ff, date, value_f)]

# fill in 0's for missing dates 

CJ(ff = unique(dat_stats_class$ff),
   value_f = unique(dat_stats_class$value_f),
   date = seq(min(dat_stats$date), max(dat_stats$date), by = "day")) %>% 
  merge(dat_stats_class, all.x = T) -> dat_stats_class_full
# with(dat_stats_class_full, table(ff, value_f, is.na(count_frac)))

# dat_stats_class_full[is.na(count_frac), count_frac := 0]

# -> some missing dates... not filling with 0



## monhtly average clouds ####

# dat_stats_class_full[value_f == "clouds", 
dat_stats_class[value_f == "clouds", 
                     .(count_frac = mean(count_frac)),
                     .(ff, month(date), value_f)] -> dat_plot

dat_plot[, hydro_month := factor(month.abb[month], levels = month.abb[c(10:12, 1:9)])]


dat_plot %>% 
  ggplot(aes(hydro_month, count_frac, group = ff, colour = ff))+
  geom_point()+
  geom_line()+
  scale_y_continuous(labels = scales::percent_format())+
  theme_bw()+
  xlab(NULL)+
  ylab("Average cloud fraction per image")


dat_stats_class[value_f == "clouds" & !startsWith(ff, "04") & !startsWith(ff, "05")] %>% 
  ggplot(aes(count_frac))+
  geom_histogram(boundary = 0)+
  scale_x_continuous(labels = scales::percent_format())+
  facet_wrap(~ ff, ncol = 1)+
  theme_bw()+
  xlab("Cloud fraction per image")+
  ylab("Count")

dat_stats_class[value_f == "clouds" & (startsWith(ff, "04") | startsWith(ff, "05"))] %>% 
  ggplot(aes(count_frac))+
  geom_histogram(boundary = 0, bins = 100)+
  scale_x_continuous(labels = scales::percent_format())+
  facet_wrap(~ ff, ncol = 1)+
  theme_bw()+
  xlab("Cloud fraction per image")+
  ylab("Count")



# sca ---------------------------------------------------------------------



dat_stats_class[, month_f := factor(month.abb[month(date)],
                                         levels = month.abb[c(10:12, 1:9)])]
dat_stats_class[, year := year(date)]
dat_stats_class[, hydro_year := year]
dat_stats_class[month(date) <= 9, hydro_year := hydro_year - 1L]
dat_stats_class[, .N, hydro_year]

dat_plot_year <- dat_stats_class[value_f == "snow" & month_f %in% c("Jan") & 
                                   hydro_year %in% c(2000:2018),
                                 .(ncell_snow = mean(count_sum)),
                                 .(month_f, year)]

# get real area
cellsize_km2 <- prod(res(rr)/1000) # km2
dat_plot_year[, area_snow := ncell_snow * cellsize_km2]

dat_plot_year %>% 
  ggplot(aes(year, area_snow))+
  geom_line()+
  theme_bw()+
  xlab(NULL)+
  ylab(expression(paste("Snow covered area [", km^2, "]")))




# ## by region ####
# 
# all_files_region <- list.files("data/modis-gapfill-stats/", full.names = T, pattern = "^reg")
# 
# dat_stats_region <- foreach(
#   i_file = all_files_region,
#   .final = rbindlist
# ) %do% {
#   
#   dat <- fread(i_file)
#   
#   dat[, .(ff = basename(i_file) %>% gsub(".csv", "", .),
#           region,
#           date = as.Date(ymd_hms(file)),
#           value, 
#           count)]
#   
# }
# 
# 
# dat_stats_region[, count_frac := count / ncell]
# dat_stats_region[, value_f := fct_collapse(as.character(value),
#                                            nodata_water = c("0", "5"),
#                                            snow = "1",
#                                            land = "2",
#                                            clouds = "3")]
# dat_stats_region[, region_f := factor(region)]
# dat_stats_region[, region_f := forcats::fct_recode(region_f, 
#                                                    SW = "1", NW = "2", NE = "3", SE = "4")]
# 
# dat_stats_region_class <- dat_stats_region[, 
#                                            .(count_frac = sum(count_frac),
#                                              count_sum = sum(count)),
#                                            .(ff, region_f, date, value_f)]
# 
# 
# # ** sca ---------------------------------------------------------------------
# 
# 
# 
# dat_stats_region_class[, month_f := factor(month.abb[month(date)],
#                                     levels = month.abb[c(10:12, 1:9)])]
# dat_stats_region_class[, year := year(date)]
# dat_stats_region_class[, hydro_year := year]
# dat_stats_region_class[month(date) <= 9, hydro_year := hydro_year - 1L]
# dat_stats_region_class[, .N, hydro_year]
# 
# dat_plot_year_region <- dat_stats_region_class[value_f == "snow" & month_f %in% c("Jan") & 
#                                    hydro_year %in% c(2000:2018),
#                                  .(ncell_snow = mean(count_sum)),
#                                  .(month_f, year, region_f)]
# 
# dat_plot_year_region[, region_f2 := fct_relevel(region_f, c("NW", "NE", "SW", "SE"))]
# 
# # get real area
# cellsize_km2 <- prod(res(rr)/1000) # km2
# dat_plot_year_region[, area_snow := ncell_snow * cellsize_km2]
# 
# dat_plot_year_region %>% 
#   ggplot(aes(year, area_snow))+
#   geom_line()+
#   facet_wrap(~region_f2)+
#   theme_bw()+
#   xlab(NULL)+
#   ylab(expression(paste("Snow covered area [", km^2, "]")))
