# viz and numbers downscaled upscaled modis


library(data.table)
library(magrittr)
library(fs)
library(foreach)
library(forcats)

library(ggplot2)
library(scico)

library(flextable)
library(officer)


dat_summ <- readRDS("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/ds/eval_upscaled_modis/02-summary-daily-elev-th.rds")


# data prep ---------------------------------------------------------------

dat_summ2 <- dat_summ[value_orig %in% c(1, 2) & !is.na(th_level)]
dat_summ2[, orig_fct := factor(ifelse(value_orig == 1, "snow", "land"))]
dat_summ2[, ds_fct := factor(ifelse(value_ds == 1, "snow", "land"))]
dat_summ2[, c("elev_min", "elev_max") := tstrsplit(elev_fct, "\\(|\\]|,", 
                                                   keep = 2:3, 
                                                   type.convert = T)]
dat_summ2[, elev_middle := (elev_min+elev_max)/2]


# overall true false

dat_tf <- dat_summ2[, 
                    .(n_true = sum(N[orig_fct == ds_fct]),
                      n_false = sum(N[orig_fct != ds_fct]),
                      n_total = sum(N)),
                    # .(elev_middle, month(date), th_level)]
                    .(elev_middle, month(date))]
dat_tf[, frac_true := n_true / n_total]
dat_tf[, frac_false := n_false / n_total]
dat_tf[, month_fct := mitmatmisc::month_fct(month, 8)]



# detailed
# pos = snow, neg = land, orig = truth
dat_xt <- dat_summ2[, 
                    .(n_tp = sum(N[orig_fct == "snow" & ds_fct == "snow"]),
                      n_tn = sum(N[orig_fct == "land" & ds_fct == "land"]),
                      n_fp = sum(N[orig_fct == "land" & ds_fct == "snow"]),
                      n_fn = sum(N[orig_fct == "snow" & ds_fct == "land"]),
                      n_total = sum(N)),
                    # .(elev_middle, month(date), th_level)]
                    .(elev_middle, month(date))]
dat_xt[, frac_tp := n_tp / n_total]
dat_xt[, frac_tn := n_tn / n_total]
dat_xt[, frac_fp := n_fp / n_total]
dat_xt[, frac_fn := n_fn / n_total]

dat_xt[, month_fct := mitmatmisc::month_fct(month, 8)]




# plot --------------------------------------------------------------------


dat_plot <- dat_tf %>% 
  melt(id.vars = c("elev_middle", "month", "month_fct"),
       measure.vars = c("n_true", "n_false"))

dat_plot[, variable_fct := fct_recode(variable,
                                      "correct" = "n_true", 
                                      "erroneous" = "n_false")]

# cols <- c("#0571b0", "#ca0020")
cols <- c("#67a9cf", "#ef8a62")

gg <- dat_plot %>% 
  ggplot(aes(elev_middle, value, fill = variable_fct))+
  geom_col(width = 100)+
  facet_wrap(~month_fct)+
  scale_fill_manual("Downscaling..", values = cols)+
  theme_bw()+
  xlab("Elevation [m]")+
  ylab("Number")



ggsave(gg,
       filename = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/02-revision/bias-past/ds-modis-hist.png",
       width = 12, height = 6, units = "in")



# plot summary metrics ---------------------------------------------------------


gg2 <- dat_xt[, .(
  sens = sum(n_tp) / sum(n_tp + n_fn),
  spec = sum(n_tn) / sum(n_tn + n_fp),
  ppv = sum(n_tp) / sum(n_tp + n_fp),
  npv = sum(n_tn) / sum(n_tn + n_fn),
  acc = sum(n_tp + n_tn) / sum(n_tp + n_fp + n_tn + n_fn)
), .(elev_middle, month_fct)] %>% 
  ggplot(aes(elev_middle))+
  geom_line(aes(y = acc))+
  scale_y_continuous(labels = scales::percent_format())+
  facet_wrap(~month_fct)+
  theme_bw()+
  xlab("Elevation [m]")+
  ylab("Downscaling accuracy")


ggsave(gg2,
       filename = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/02-revision/bias-past/ds-modis-acc.png",
       width = 12, height = 6, units = "in")




# table -------------------------------------------------------------------


dat_xt[, .(
  sens = sum(n_tp) / sum(n_tp + n_fn),
  spec = sum(n_tn) / sum(n_tn + n_fp),
  ppv = sum(n_tp) / sum(n_tp + n_fp),
  npv = sum(n_tn) / sum(n_tn + n_fn),
  acc = sum(n_tp + n_tn) / sum(n_tp + n_fp + n_tn + n_fn)
  )] %>% 
  flextable() %>% 
  colformat_double(digits = 3) -> ft1


dat_xt[, .(
  sens = sum(n_tp) / sum(n_tp + n_fn),
  spec = sum(n_tn) / sum(n_tn + n_fp),
  ppv = sum(n_tp) / sum(n_tp + n_fp),
  npv = sum(n_tn) / sum(n_tn + n_fn),
  acc = sum(n_tp + n_tn) / sum(n_tp + n_fp + n_tn + n_fn)
), .(elev_middle, month_fct)] %>% 
  dcast(elev_middle ~ month_fct, value.var = "acc") %>% 
  flextable() %>% 
  colformat_double(digits = 3, j = 2:13) %>% 
  vline(j = 1, border = fp_border()) %>% 
  fontsize(size = 7, part = "all") %>% 
  autofit(0, 0) -> ft2



read_docx() %>% 
  body_add_flextable(ft1) %>% 
  body_add_par(" new table ") %>% 
  body_add_flextable(ft2) %>% 
  print(target = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/02-revision/bias-past/table-ds-modis-acc.docx")

  

# numbers -----------------------------------------------------------------

rr <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/modis_scd/clim/2000-08-01_2020-07-31.tif")
ncell(rr)
dat_summ[, sum(N), date]
dat_summ[, sum(N)]



