# scale issues in scf/scd clim bias spatial and elevation

library(stars)
library(raster)
library(data.table)
library(magrittr)
library(ggplot2)
library(cowplot)
library(foreach)
library(fs)
library(graticule)


elev_int <- 100 


# modis hr data -----------------------------------------------------------


rr_elev <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_altitude_laea.tif")
rr_elev[rr_elev < -30000] <- NA

rr_water <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/00_aux/eurac_modis_water_mask_laea.tif")
rr_elev[rr_water == 1] <- NA

# rr_modis <- raster("/mnt/CEPH_PROJECTS/SNOW_3rd_gen/CLOUDREMOVAL/v1.1/clim_SCD/2000-10-01_2019-09-30.tif")
rr_modis <- raster("/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/modis_scd/clim/2000-08-01_2020-07-31.tif")


dat_modis <- data.table(icell = 1:ncell(rr_modis),
                        scd = rr_modis[],
                        elev = rr_elev[])

dat_modis[, elev_f := cut(elev, breaks = c(seq(-200, 4800, by = elev_int)), dig.lab = 5)]

dat_modis2 <- dat_modis[!is.na(scd), 
                        .(scd = mean(scd),
                          nn = .N),
                        .(elev_f)]



## lr ----------------------------------------------------------------------

rr_modis_lr <- aggregate(rr_modis,
                         fact = c(37, 53),
                         fun = mean,
                         na.rm = T)
rr_elev_lr <- aggregate(rr_elev,
                         fact = c(37, 53),
                         fun = mean,
                         na.rm = T)

dat_modis_lr <- data.table(icell = 1:ncell(rr_modis_lr),
                           scd = rr_modis_lr[],
                           elev = rr_elev_lr[])

dat_modis_lr[, elev_f := cut(elev, breaks = c(seq(-200, 3600, by = elev_int), 4200), dig.lab = 5)]

dat_modis_lr2 <- dat_modis_lr[!is.na(scd), 
                              .(scd = mean(scd),
                                nn = .N),
                              .(elev_f)]


## lr2hr for maps --------------------------------------------------------------


mat_lr <- raster::as.matrix(rr_modis_lr)
mat_lr2hr <- mat_lr[rep(1:55, each = 53),
                    rep(1:133, each = 37)]
mat_lr2hr <- mat_lr2hr[1:2867, 1:4901]
rr_modis_lr2hr <- raster(rr_modis)
rr_modis_lr2hr[] <- mat_lr2hr

rs_hr <- st_as_stars(rr_modis)
rs_lr <- st_as_stars(rr_modis_lr)
rs_diff_abs <- st_as_stars(rr_modis_lr2hr - rr_modis)
rs_diff_rel <- st_as_stars((rr_modis_lr2hr - rr_modis)/rr_modis)

# remove extreme relative from plot
rs_diff_rel[rs_diff_rel > 1] <- 1
# rs_diff_rel[rs_diff_rel > 10] <- NA

# elev bias hr to lr ------------------------------------------------------

dat_plot <- rbind(cbind(dat_modis2, ff = "high res (250m)"),
                  cbind(dat_modis_lr2, ff = "low res (0.11 deg)"))
dat_plot[, elev := tstrsplit(elev_f, ",") %>% sapply(readr::parse_number) %>% rowMeans]

setorder(dat_plot, ff, elev)

dat_plot[, nn_total_grp := sum(nn), .(ff)]
dat_plot[, nn_frac := nn / nn_total_grp]

dat_plot_diff <- dat_plot[nn > 1] %>% 
  dcast(elev ~ ff, value.var = "scd")

dat_plot_diff[, diff_abs := `low res (0.11 deg)` - `high res (250m)`]
dat_plot_diff[, diff_rel := (`low res (0.11 deg)` - `high res (250m)`) / `high res (250m)`]



gg_scd <- dat_plot[nn > 1] %>% 
  ggplot(aes(scd, elev, colour = ff))+
  geom_path()+
  scale_color_brewer("", palette = "Set1")+
  theme_bw()+
  theme(legend.position = c(0.8, 0.2),
        legend.background = element_rect(colour = "black"),
        legend.title = element_blank())+
  xlab("Average SCD [d]")+
  ylab("Elevation [m]")


gg_elev <- dat_plot[nn > 1] %>% 
  ggplot(aes(nn_frac, elev, colour = ff))+
  geom_path(aes(group = ff))+
  scale_color_brewer("", palette = "Set1")+
  scale_x_continuous(labels = scales::percent_format())+
  theme_bw()+
  theme(legend.position = c(0.8, 0.8),
        legend.background = element_rect(colour = "black"),
        legend.title = element_blank())+
  xlab("Fraction of pixels/grid cells")+
  ylab("Elevation [m]")



gg_diff_abs <- dat_plot_diff %>% 
  ggplot(aes(diff_abs, elev))+
  geom_vline(xintercept = 0, linetype = "dashed")+
  geom_path()+
  geom_point()+
  theme_bw()+
  xlab("Absolute difference in SCD (low - high res) [d]")+
  ylab("Elevation [m]")


gg_diff_rel <- dat_plot_diff %>% 
  ggplot(aes(diff_rel, elev))+
  geom_vline(xintercept = 0, linetype = "dashed")+
  geom_path()+
  geom_point()+
  scale_x_continuous(labels = scales::percent_format())+
  theme_bw()+
  xlab("Relative difference in SCD (low - high res)")+
  ylab("Elevation [m]")








gg_comb <- plot_grid(gg_scd, gg_elev,
                     gg_diff_abs, gg_diff_rel,
                     align = "hv",
                     labels = c("(a)", "(b)", "(c)", "(d)"))


ggsave(filename = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/02-revision/scale-issues-01-elev.png",
       gg_comb,
       width = 9,
       height = 7)




# maps --------------------------------------------------------------------

ds_stars <- 5

prj <- st_crs(rs_hr)$proj4string

lons <- seq(10, 12, by = 2)
lats <- seq(46, 48, by = 1)
xl <- range(lons) + c(-0.4, 0.4)
yl <- range(lats) + c(-0.4, 0.4)
grat <- graticule(lons, lats, proj = prj, xlim = xl, ylim = yl)
labs <- graticule_labels(lons, lats, xline = min(xl), yline = max(yl), proj = prj)



gg_map_hr <- 
  ggplot()+
  geom_sf(data = st_as_sf(grat), colour = NA)+
  geom_stars(data = rs_hr, downsample = ds_stars)+
  scale_fill_scico("SCD [d]", limits = c(0, 366),
                   palette = "davos", na.value = NA, direction = 1)+
  coord_sf(expand = F)+
  theme_minimal()+
  xlab(NULL)+ylab(NULL)+
  scale_x_continuous(labels = function(x) paste0(x, "° E"))+
  scale_y_continuous(labels = function(x) paste0(x, "° N"))


gg_map_lr <-
  ggplot()+
    geom_sf(data = st_as_sf(grat), colour = NA)+
    geom_stars(data = rs_lr)+
    scale_fill_scico("SCD [d]", limits = c(0, 366),
                     palette = "davos", na.value = NA, direction = 1)+
    coord_sf(expand = F)+
    theme_minimal()+
    xlab(NULL)+ylab(NULL)+
    scale_x_continuous(labels = function(x) paste0(x, "° E"))+
    scale_y_continuous(labels = function(x) paste0(x, "° N"))


  
gg_map_diff_abs <-
  ggplot()+
    geom_sf(data = st_as_sf(grat), colour = NA)+
    geom_stars(data = rs_diff_abs, downsample = ds_stars)+
    scale_fill_scico("dSCD [d]", 
                     palette = "vik", na.value = NA, direction = -1,
                     rescaler = scales::rescale_mid)+
    coord_sf(expand = F)+
    theme_minimal()+
    xlab(NULL)+ylab(NULL)+
    scale_x_continuous(labels = function(x) paste0(x, "° E"))+
    scale_y_continuous(labels = function(x) paste0(x, "° N"))


gg_map_diff_rel <-
  ggplot()+
    geom_sf(data = st_as_sf(grat), colour = NA)+
    geom_stars(data = rs_diff_rel*100, downsample = ds_stars)+
    scale_fill_scico("dSCD [%]", 
                     palette = "vik", na.value = NA, direction = -1,
                     rescaler = scales::rescale_mid)+
    coord_sf(expand = F)+
    theme_minimal()+
    xlab(NULL)+ylab(NULL)+
    scale_x_continuous(labels = function(x) paste0(x, "° E"))+
    scale_y_continuous(labels = function(x) paste0(x, "° N"))
  




gg_map_comb <- plot_grid(gg_map_hr, gg_map_lr,
                         gg_map_diff_abs, gg_map_diff_rel,
                         align = "hv",
                         labels = c("(a)", "(b)", "(c)", "(d)"))


ggsave(filename = "/mnt/CEPH_PROJECTS/CLIRSNOW/data_bc/paper-fig-table/02-revision/scale-issues-02-map.png",
       gg_map_comb,
       width = 12,
       height = 7)
